#!/usr/local/bin/bash
## bootparse.sh 1.0
## Copyright (C) 2011 Isaac Schemm <isaacschemm@gmail.com>
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
##
## The full text of the GNU GPL, versions 2 or 3, can be found at
## <http://www.gnu.org/copyleft/gpl.html>, on the NetbootCD site at
## <http://netbootcd.tuxfamily.org>, or on the CD itself.

mainFunction() {
	INPUTFILE=$1
	CDMOUNTPATH=/mnt/nbcd

	#CURRENT is a counter of array indices. The first element is stored at 1.
	CURRENT=0
	while read i;do
		FIRST=$(echo $i|cut -f1 -d ' '|tr '[:upper:]' '[:lower:]')
		REST=$(echo $i|cut -f2- -d ' ')
		if [ "$FIRST" = "label" ];then
			CURRENT=$(($CURRENT+1))
			labels[$CURRENT]="$REST"
			menulabels[$CURRENT]="$REST"
		elif [ "$FIRST" = "menu" ];then
			if [ "$(echo $REST|cut -f1 -d ' '|tr '[:upper:]' '[:lower:]')" = "label" ];then
				menulabels[$CURRENT]="$(echo $REST|cut -f2- -d ' ')"
			fi
		elif [ "$FIRST" = "kernel" ] || [ "$FIRST" = "linux" ];then
			kernels[$CURRENT]="$REST"
		elif [ "$FIRST" = "append" ];then
			appends[$CURRENT]="$REST"
		elif [ "$FIRST" = "initrd" ];then
			initrds[$CURRENT]="$REST"
		fi
	done < $INPUTFILE

	#Extract the initrd path from the append string
	SIZE=$CURRENT
	for i in $(seq 1 $SIZE);do
		APPENDS="${appends[$i]}"
		NEWAPPENDS=""
		for j in $APPENDS;do
			if echo $j|grep -q '^initrd=';then
				initrds[$i]=$(echo $j|tail -c +8)
			else
				NEWAPPENDS="${NEWAPPENDS} ${j}"
			fi
		done
		appends[$i]="$NEWAPPENDS"
	done

	#Fix absolute paths for kernels/initrds
	for i in $(seq 1 $SIZE);do
		if [ "$(echo ${kernels[$i]} | cut -c 1)" = "/" ];then
			kernels[$i]="/mnt/nbcd"${kernels[$i]}
		fi
		if [ "$(echo ${initrds[$i]} | cut -c 1)" = "/" ];then
			initrds[$i]="/mnt/nbcd"$(echo ${initrds[$i]}|sed -e 's^,/^,/mnt/nbcd/^g')
		fi
	done

	#Launch dialog
	echo "dialog --menu \"$(readlink -f $1|sed -e 's^/mnt/nbcd^^g')\" 0 0 0 \\" > /tmp/dialog-launcher
	for i in $(seq 1 $SIZE);do
		if [ -n "${kernels[$i]}" ];then #don't show labels without a "kernel" or "linux" command
			echo "${labels[$i]} \"${menulabels[$i]}\" \\" >> /tmp/dialog-launcher
		fi
	done
	bash /tmp/dialog-launcher 2> /tmp/output
	CHOICE=$(cat /tmp/output)
	rm /tmp/output /tmp/dialog-launcher

	#Build launch command
	for i in $(seq 1 $SIZE);do
		if [ "$CHOICE" = "${labels[$i]}" ];then
			LAUNCH="-l "${kernels[$i]}
			if [ -n "${initrds[$i]}" ];then
				if echo ${initrds[$i]}|grep -q ,;then
					#Combine initrds by storing contents in /tmp if necessary
					mkdir /tmp/newinit
					for j in $(echo ${initrds[$i]}|sed -e 's/--initrd=//g' -e 's/,/ /g');do
						INIT=$(readlink -f $j)
						echo "Copying $INIT..."
						cd /tmp/newinit
						gzip -cd $INIT|cpio --quiet -id 2> /dev/null
						echo $?
						cd - > /dev/null
					done
					cd /tmp/newinit
					echo "Making new initrd..."
					(find .|cpio --quiet -o -H newc>/tmp/initrd)
					cd - > /dev/null
					rm -r /tmp/newinit
					LAUNCH="${LAUNCH} --initrd=/tmp/initrd"
				else
					LAUNCH="${LAUNCH} --initrd=$(echo ${initrds[$i]})"
				fi
			fi
			if [ -n "${appends[$i]}" ];then
				LAUNCH="${LAUNCH} --append=\"${appends[$i]}\""
			fi
		break
		fi
	done

	if [ -d /home/tc ];then #Stops kexec from running if you're not in Tiny Core (e.g. NetbootCD)
		kexec $LAUNCH > /dev/null || echo "Could not load: \"kexec $LAUNCH\""
		sync
		kexec -e
	fi
}

searchForCFG() {
	FOLDER=$(echo $1|sed -e 's/\.cfg//g') #folder to look in (i.e. "isolinux" for isolinux.cfg)
	if [ -f /mnt/nbcd/boot/$FOLDER/$1 ];then
		cd /mnt/nbcd/boot/$FOLDER #We have to cd to the right directory so relative paths in the .cfg file work. Absolute paths will be fixed by read-cfg.sh.
		mainFunction $1           #Leaves this script and goes to read-cfg.sh, which will handle the kexec itself.
	elif [ -f /mnt/nbcd/$FOLDER/$1 ];then
		cd /mnt/nbcd/$FOLDER
		mainFunction $1
	elif [ -f /mnt/nbcd/$1 ];then
		cd /mnt/nbcd
		mainFunction $1
	fi
}

dialog --backtitle "NetbootCD - Load Script" --menu "File to search for:" 20 70 13 \
nbcd.cfg "A file intended for NetbootCD (SYSLINUX syntax)" \
isolinux.cfg "ISOLINUX cfg file (for CDs)" \
syslinux.cfg "SYSLINUX cfg file (for FAT drives)" \
extlinux.cfg "EXTLINUX cfg file (for ext2+ drives)" 2> /tmp/nb-file
FILE=$(cat /tmp/nb-file)
rm /tmp/nb-file

umount /mnt/nbcd 2> /dev/null || true
mkdir -p /mnt/nbcd
if mount /dev/cdrom /mnt/nbcd;then #Try CD first
	searchForCFG $FILE #find the file, then run read-cfg.sh to load it
fi
umount /mnt/nbcd 2> /dev/null || true

fdisk -l | grep '^/' | awk '{print $1}' > /tmp/fdisk-list #Get list of partitions
for i in $(cat /tmp/fdisk-list);do
	if mount $i /mnt/nbcd;then #try each partition
		searchForCFG $FILE #find the file, then run read-cfg.sh to load it
	else
		umount /mnt/nbcd 2> /dev/null || true
	fi
done
echo "Could not find $VERSION!"
exit 1

